﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO.Ports;


public struct AD101DEVICEPARAMETER
{
    public int nRingOn;
    public int nRingOff;
    public int nHookOn;
    public int nHookOff;
    public int nStopCID;
    public int nNoLine;			// Add this parameter in new AD101(MCU Version is 6.0)
}

namespace CHC
{
    public partial class FormMain : Form
    {
        private string COMPort;
        private int MaxListviewItems;
        public static SerialPort serialPort = new SerialPort();

        #region Artech AD101 functions

        [DllImport("AD101Device.dll", EntryPoint = "AD101_InitDevice")]
        public static extern int AD101_InitDevice(int hWnd);

        [DllImport("AD101Device.dll", EntryPoint = "AD101_GetDevice")]
        public static extern int AD101_GetDevice();

        [DllImport("AD101Device.dll", EntryPoint = "AD101_GetCPUVersion", CharSet = CharSet.Ansi)]
        public static extern int AD101_GetCPUVersion(int nLine, StringBuilder szCPUVersion);

        // Start reading cpu id of device 
        [DllImport("AD101Device.dll", EntryPoint = "AD101_ReadCPUID")]
        public static extern int AD101_ReadCPUID(int nLine);

        // Get readed cpu id of device 
        [DllImport("AD101Device.dll", EntryPoint = "AD101_GetCPUID", CharSet = CharSet.Ansi)]
        public static extern int AD101_GetCPUID(int nLine, StringBuilder szCPUID);



        // Get caller id number  
        [DllImport("AD101Device.dll", EntryPoint = "AD101_GetCallerID", CharSet = CharSet.Ansi)]
        public static extern int AD101_GetCallerID(int nLine, StringBuilder szCallerIDBuffer, StringBuilder szName, StringBuilder szTime);

        // Get dialed number 
        [DllImport("AD101Device.dll", EntryPoint = "AD101_GetDialDigit", CharSet = CharSet.Ansi)]
        public static extern int AD101_GetDialDigit(int nLine, StringBuilder szDialDigitBuffer);


        // Get collateral phone dialed number 
        [DllImport("AD101Device.dll", EntryPoint = "AD101_GetCollateralDialDigit", CharSet = CharSet.Ansi)]
        public static extern int AD101_GetCollateralDialDigit(int nLine, StringBuilder szDialDigitBuffer);



        // Get last line state 
        [DllImport("AD101Device.dll", EntryPoint = "AD101_GetState")]
        public static extern int AD101_GetState(int nLine);

        // Get ring count
        [DllImport("AD101Device.dll", EntryPoint = "AD101_GetRingIndex")]
        public static extern int AD101_GetRingIndex(int nLine);

        // Get talking time
        [DllImport("AD101Device.dll", EntryPoint = "AD101_GetTalkTime")]
        public static extern int AD101_GetTalkTime(int nLine);


        [DllImport("AD101Device.dll", EntryPoint = "AD101_GetParameter")]
        public static extern int AD101_GetParameter(int nLine, ref AD101DEVICEPARAMETER tagParameter);

        [DllImport("AD101Device.dll", EntryPoint = "AD101_ReadParameter")]
        public static extern int AD101_ReadParameter(int nLine);

        // Set systematic parameter  
        [DllImport("AD101Device.dll", EntryPoint = "AD101_SetParameter")]
        public static extern int AD101_SetParameter(int nLine, ref AD101DEVICEPARAMETER tagParameter);

        // Free devices 

        [DllImport("AD101Device.dll", EntryPoint = "AD101_FreeDevice")]
        public static extern void AD101_FreeDevice();

        // Get current AD101 device count
        [DllImport("AD101Device.dll", EntryPoint = "AD101_GetCurDevCount")]
        public static extern int AD101_GetCurDevCount();

        // Change handle of window that uses to receive message
        [DllImport("AD101Device.dll", EntryPoint = "AD101_ChangeWindowHandle")]
        public static extern int AD101_ChangeWindowHandle(int hWnd);

        // Show or don't show collateral phone dialed number
        [DllImport("AD101Device.dll", EntryPoint = "AD101_ShowCollateralPhoneDialed")]
        public static extern void AD101_ShowCollateralPhoneDialed(bool bShow);

        // Control led 
        [DllImport("AD101Device.dll", EntryPoint = "AD101_SetLED")]
        public static extern int AD101_SetLED(int nLine, int enumLed);


        // Control line connected with ad101 device to busy or idel
        [DllImport("AD101Device.dll", EntryPoint = "AD101_SetBusy")]
        public static extern int AD101_SetBusy(int nLine, int enumLineBusy);

        // Set line to start talking than start timer
        [DllImport("AD101Device.dll", EntryPoint = "AD101_SetLineStartTalk")]
        public static extern int AD101_SetLineStartTalk(int nLine);


        // Set time to start talking after dialed number 
        [DllImport("AD101Device.dll", EntryPoint = "AD101_SetDialOutStartTalkingTime")]
        public static extern int AD101_SetDialOutStartTalkingTime(int nSecond);


        // Set ring end time
        [DllImport("AD101Device.dll", EntryPoint = "AD101_SetRingOffTime")]
        public static extern int AD101_SetRingOffTime(int nSecond);



        public const int MCU_BACKID = 0x07;	// Return Device ID
        public const int MCU_BACKSTATE = 0x08;	// Return Device State
        public const int MCU_BACKCID = 0x09;		// Return Device CallerID
        public const int MCU_BACKDIGIT = 0x0A;	// Return Device Dial Digit
        public const int MCU_BACKDEVICE = 0x0B;	// Return Device Back Device ID
        public const int MCU_BACKPARAM = 0x0C;	// Return Device Paramter
        public const int MCU_BACKCPUID = 0x0D;	// Return Device CPU ID
        public const int MCU_BACKCOLLATERAL = 0x0E;		// Return Collateral phone dialed
        public const int MCU_BACKDISABLE = 0xFF;		// Return Device Init
        public const int MCU_BACKENABLE = 0xEE;
        public const int MCU_BACKMISSED = 0xAA;		// Missed call 
        public const int MCU_BACKTALK = 0xBB;		// Start Talk

        // LED Status 
        enum LEDTYPE
        {
            LED_CLOSE = 1,
            LED_RED,
            LED_GREEN,
            LED_YELLOW,
            LED_REDSLOW,
            LED_GREENSLOW,
            LED_YELLOWSLOW,
            LED_REDQUICK,
            LED_GREENQUICK,
            LED_YELLOWQUICK,
        };
        //////////////////////////////////////////////////////////////////////////////////////////////

        // Line Status 
        enum ENUMLINEBUSY
        {
            LINEBUSY = 0,
            LINEFREE,
        };


        public const int HKONSTATEPRA = 0x01; // hook on pr+  HOOKON_PRA
        public const int HKONSTATEPRB = 0x02;  // hook on pr-  HOOKON_PRR
        public const int HKONSTATENOPR = 0x03;  // have pr  HAVE_PR
        public const int HKOFFSTATEPRA = 0x04;   // hook off pr+  HOOKOFF_PRA
        public const int HKOFFSTATEPRB = 0x05;  // hook off pr-  HOOKOFF_PRR
        public const int NO_LINE = 0x06; // no line  NULL_LINE
        public const int RINGONSTATE = 0x07;  // ring on  RING_ON
        public const int RINGOFFSTATE = 0x08;  // ring off RING_OFF
        public const int NOHKPRA = 0x09; // NOHOOKPRA= 0x09, // no hook pr+
        public const int NOHKPRB = 0x0a; // NOHOOKPRR= 0x0a, // no hook pr-
        public const int NOHKNOPR = 0x0b; // NOHOOKNPR= 0x0b, // no hook no pr

        public const int WM_USBLINEMSG = 1024 + 180;

        #endregion
        public FormMain()
        {
            InitializeComponent();

            int width = Screen.PrimaryScreen.WorkingArea.Width;
            int height = Screen.PrimaryScreen.WorkingArea.Height;

            this.Left = width - this.Width;
            this.Top = 0;
            this.Height = height;
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            listViewPhone.Columns.Clear();
            listViewPhone.Columns.Add("Number", 150, HorizontalAlignment.Center);
            listViewPhone.Columns.Add("Date/Time", 230, HorizontalAlignment.Center);

            if (AD101_InitDevice(Handle.ToInt32()) == 0)
            {
                return;
            }

            AD101_GetDevice();
            AD101_SetDialOutStartTalkingTime(3);
            AD101_SetRingOffTime(7);

            COMPort = Properties.Settings.Default.COMPort;
            MaxListviewItems = Properties.Settings.Default.MaxListviewItems;

            serialPort.PortName = COMPort;
            serialPort.BaudRate = 9600;
            serialPort.Open();
        }

        private void FormMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            serialPort.Close();
            AD101_FreeDevice();
        }

        private void OnCallerID(string nmbr)
        {
            string sDate = "";
            string sTime = "";
            string sNumber = "";
            DateTime datetime = DateTime.Now;

            sDate = "DATE ";
            sDate += datetime.Month.ToString();
            sDate += datetime.Day.ToString();
            sDate += "\n\r";

            sTime = "TIME ";
            sTime += datetime.Hour.ToString();
            sTime += datetime.Minute.ToString();
            sTime += "\n\r";

            sNumber = "NMBR ";
            sNumber += nmbr;
            sNumber += "\n\r";


            ListViewItem item = new ListViewItem(new[] { nmbr, datetime.ToString() });
            listViewPhone.Items.Add(item);

            if(listViewPhone.Items.Count > MaxListviewItems)
            {
                listViewPhone.Items[1].Remove();
            }

            serialPort.Write("RING\n\r");
            serialPort.Write(sDate);
            serialPort.Write(sTime);
            serialPort.Write(sNumber);
        }

        private void OnDeviceMsg(IntPtr wParam, IntPtr Lparam)
        {
            int nMsg = new int();
            int nLine = new int();

            nMsg = wParam.ToInt32() % 65536;
            nLine = wParam.ToInt32() / 65536;

            switch (nMsg)
            {
                case MCU_BACKDISABLE:
                    break;

                case MCU_BACKENABLE:
                    // TODO: Add indicator to mainform so activity on 4 devices can be seen
                    //listViewPhone.Items[nLine].SubItems[1].Text = "Enable";
                    break;

                case MCU_BACKID:
                    break;

                case MCU_BACKCID:
                    {
                        StringBuilder szCallerID = new StringBuilder(128);
                        StringBuilder szName = new StringBuilder(128);
                        StringBuilder szTime = new StringBuilder(128);

                        int nLen = AD101_GetCallerID(nLine, szCallerID, szName, szTime);
                        //listViewPhone.Items[nLine].SubItems[4].Text = szCallerID.ToString();
                        this.OnCallerID(szCallerID.ToString());
                    }
                    break;

                case MCU_BACKSTATE:
                    {
                        switch (Lparam.ToInt32())
                        {
                            case HKONSTATEPRA:
                                break;

                            case HKONSTATEPRB:
                                break;

                            case HKONSTATENOPR:
                                break;

                            case HKOFFSTATEPRA:
                                {
                                    StringBuilder szCallerID = new StringBuilder(128);
                                    StringBuilder szName = new StringBuilder(128);
                                    StringBuilder szTime = new StringBuilder(128);

                                    if (AD101_GetCallerID(nLine, szCallerID, szName, szTime) < 1 || AD101_GetRingIndex(nLine) < 1)
                                    {
                                        this.OnCallerID(szCallerID.ToString());
                                    }

                                }
                                break;

                            case HKOFFSTATEPRB:
                                {

                                    StringBuilder szCallerID = new StringBuilder(128);
                                    StringBuilder szName = new StringBuilder(128);
                                    StringBuilder szTime = new StringBuilder(128);

                                    if (AD101_GetCallerID(nLine, szCallerID, szName, szTime) < 1 || AD101_GetRingIndex(nLine) < 1)
                                    {
                                        this.OnCallerID(szCallerID.ToString());
                                    }
                                }
                                break;

                            case NO_LINE:
                                {
                                    StringBuilder szCallerID = new StringBuilder(128);
                                    StringBuilder szName = new StringBuilder(128);
                                    StringBuilder szTime = new StringBuilder(128);

                                    if (AD101_GetCallerID(nLine, szCallerID, szName, szTime) < 1 || AD101_GetRingIndex(nLine) < 1)
                                    {
                                        this.OnCallerID(szCallerID.ToString());
                                    }
                                }
                                break;

                            case RINGONSTATE:
                                break;

                            case RINGOFFSTATE:
                                break;

                            case NOHKPRA:
                                break;

                            case NOHKPRB:
                                break;

                            case NOHKNOPR:
                                {
                                    StringBuilder szCallerID = new StringBuilder(128);
                                    StringBuilder szName = new StringBuilder(128);
                                    StringBuilder szTime = new StringBuilder(128);

                                    if (AD101_GetCallerID(nLine, szCallerID, szName, szTime) < 1 || AD101_GetRingIndex(nLine) < 1)
                                    {
                                        //listViewPhone.Items[nLine].SubItems[4].Text = "";
                                    }
                                }
                                break;

                            default:
                                break;
                        }
                    }
                    break;

                case MCU_BACKDIGIT:
                    break;


                case MCU_BACKCOLLATERAL:
                    break;

                case MCU_BACKDEVICE:
                    break;

                case MCU_BACKPARAM:
                    break;

                case MCU_BACKCPUID:
                    break;

                case MCU_BACKMISSED:
                    break;

                case MCU_BACKTALK:
                    break;

                default:
                    break;
            }
        }

        protected override void DefWndProc(ref System.Windows.Forms.Message m)
        {
            switch (m.Msg)
            {
                case WM_USBLINEMSG:
                    OnDeviceMsg(m.WParam, m.LParam);
                    break;
                default:
                    base.DefWndProc(ref m);
                    break;
            }
        }
    }
}
